Module: Traject::SolrJsonHttpxWriter::HttpxRaiseErrorPlugin
- Defined in:
- lib/traject/solr_json_httpx_writer.rb
Overview
HTTPX has really weird annoying error handling. https://honeyryderchuck.gitlab.io/httpx/wiki/Error-Handling
We add a custom plugin to make errors raise more ruby way. https://honeyryderchuck.gitlab.io/httpx/wiki/Custom-Plugins
Works as long as we're not doing multiple parallel requests in one call, which we aren't, DRY's up our error handling.
Defined Under Namespace
Modules: InstanceMethods